from functools import lru_cache
import re

from .I18N import _
from .TagsRepo import TagsRepo


@lru_cache(maxsize=1)
def get_character_tags(tags_repo: TagsRepo):
    """Zwraca zbiór nazw tagów z kategorii Character (kategoria = 4) z bazy tags.sqlite."""
    try:
        conn = tags_repo.get_conn()
        cursor = conn.cursor()
        cursor.execute("SELECT name FROM tags WHERE category = 4")
        rows = cursor.fetchall()
        conn.close()
        return {row[0] for row in rows}
    except Exception as e:
        print(_("Błąd przy pobieraniu tagów postaci:"), e)
        return set()


@lru_cache(maxsize=1)
def get_copyright_tags(tags_repo: TagsRepo):
    """Zwraca zbiór nazw tagów z kategorii Copyright (kategoria = 3) z bazy tags.sqlite."""
    try:
        conn = tags_repo.get_conn()
        cursor = conn.cursor()
        cursor.execute("SELECT name FROM tags WHERE category = 3")
        rows = cursor.fetchall()
        conn.close()
        return {row[0] for row in rows}
    except Exception as e:
        print(_("Błąd przy pobieraniu tagów copyright:"), e)
        return set()

@lru_cache(maxsize=1)
def get_artist_tags(tags_repo: TagsRepo):
    """Zwraca zbiór nazw tagów z kategorii Artist (kategoria = 1) z bazy tags.sqlite"""
    try:
        conn = tags_repo.get_conn()
        cursor = conn.cursor()
        cursor.execute("SELECT name FROM tags WHERE category = 1")
        rows = cursor.fetchall()
        conn.close()
        return {row[0] for row in rows}
    except Exception as e:
        print(_("Błąd przy pobieraniu tagów artystów:"), e)
        return set()


# Wzorce i ustawienia związane z tagami
COEFFICIENT_PATTERN = re.compile(r"^.*?(:\d+|\d+\.\d+)$")
UNESCAPED_PATTERN = re.compile(r"(?<!\\)[\(\)\[\]]+")
SLASH_PATTERN = re.compile(r"\\+(?=[\(\)](?!\^))")
WHITESPACE_PATTERN = re.compile(r"\s+|_+")
AUTO_METATAGS = [
    "absurdres",
    "high_score",
    "great_score",
    "masterpiece",
    "general",
    "sensitive",
    "questionable",
    "explicit",
    "nsfw",
    "safe",
    "dynamic_angle",
    "soft_lighting",
    "vibrant_colors",
    "cinematic_lighting",
    "detailed_background",
    "ultra-detailed",
    "wallpaper",
    "score_9",
    "score_8_up",
    "score_7_up",
    "score_6_up",
    "score_5_up",
    "score_4_up",
    "score_3_up",
    "score_2_up",
    "score_1_up",
]
TAG_FIXES = {
    "2boy": "2boys",
    "2girl": "2girls",
    "exercise": "exercising",
    "keyboard_(computer)": "computer_keyboard",
    "official_art": "official_style",
}
MULTI_TAG_FIXES = {
    "black_choker_with_heart_charm": ["black_choker", "heart_choker"],
    "orange_slit_pupils": ["orange_eyes", "slit_pupils"],
    "oversized_black_print_shirt_with_single_bare_shoulder": [
        "oversized_shirt",
        "black_shirt",
        "print_shirt",
        "single_bare_shoulder",
    ],
    "two-tone_hair_black_with_red_streaks": [
        "two-tone_hair",
        "black_hair",
        "red_streaks",
    ],
    "very_short_messy_hair": ["very_short_hair", "messy_hair"],
}


def parse_parameters(param_str, tags_repo: TagsRepo):
    """
    Funkcja do parsowania zawartości pola 'parameters' z pliku PNG.
    """
    tags = (
        param_str.split("\nNegative", 1)[0]
        .removesuffix(",")
        .replace("\n", " ")
        .split(",")
    )
    tags = set([WHITESPACE_PATTERN.sub("_", param.strip()) for param in tags])
    tags = set(
        [
            COEFFICIENT_PATTERN.sub(
                "", SLASH_PATTERN.sub("", UNESCAPED_PATTERN.sub("", tag))
            )
            for tag in tags
        ]
    )
    # Zamień nieprawidłowe tagi na poprawne
    for wrong_tag in TAG_FIXES:
        if wrong_tag in tags:
            tags.discard(wrong_tag)
            tags.add(TAG_FIXES[wrong_tag])
    for wrong_tag in MULTI_TAG_FIXES:
        if wrong_tag in tags:
            tags.discard(wrong_tag)
            tags.update(MULTI_TAG_FIXES[wrong_tag])
    # Usuń tagi automatycznie dodawane przez Kapitanbooru
    for bad_tag in AUTO_METATAGS:
        tags.discard(bad_tag)
    # Usuń tagi tekstowe
    tags = {tag for tag in tags if not tag.startswith("text:")}
    # Pobierz tagi z bazy
    character_tags = get_character_tags(tags_repo)
    copyright_tags = get_copyright_tags(tags_repo)
    artist_tags = get_artist_tags(tags_repo)

    # Dla tagów należących do kategorii Character
    for tag in list(tags):  # iterujemy po kopii zbioru
        if tag in character_tags:
            tags.discard(tag)
            tags.add("character:" + tag)
    # Dla tagów należących do kategorii Copyright
    for tag in list(tags):
        if tag in copyright_tags:
            tags.discard(tag)
            tags.add("copyright:" + tag)
    # Dla tagów należących do kategorii Artist
    artist_tags_counter = 0
    for tag in list(tags):
        if tag in artist_tags:
            tags.discard(tag)
            tags.add(tag + "_(style)")
            tags.add("style_parody")
            artist_tags_counter += 1
    if artist_tags_counter > 1:
        tags.add("multiple_style_parody")

    tags = list(tags)
    tags.sort()
    return " ".join(tags)


def process_tag(tag, tags_repo: TagsRepo):
    """
    Dla danego tagu usuwa znane prefiksy, sprawdza w tabeli tag_aliases,
    a następnie w tabeli tags – zwraca (processed_tag, status), gdzie status:
    - True: tag istnieje i jest deprecated,
    - False: tag istnieje i nie jest deprecated,
    - None: tag nie istnieje w bazie.
    """
    # Usuń prefiks, jeśli występuje
    for prefix in ["character:", "artist:", "meta:", "copyright:", "general:"]:
        if tag.startswith(prefix):
            tag_lookup = tag[len(prefix) :]
            break
    else:
        tag_lookup = tag

    # Sprawdź aliasy
    try:
        conn = tags_repo.get_conn()
        cursor = conn.cursor()
        cursor.execute("SELECT tag FROM tag_aliases WHERE alias = ?", (tag_lookup,))
        row = cursor.fetchone()
        if row:
            tag_lookup = row[0]
        conn.close()
    except Exception as e:
        print(_("Błąd podczas odczytu tag_aliases:"), e)

    # Sprawdź w tabeli tags – kolumna name nie zawiera prefiksów
    try:
        conn = tags_repo.get_conn()
        cursor = conn.cursor()
        cursor.execute("SELECT is_deprecated FROM tags WHERE name = ?", (tag_lookup,))
        row = cursor.fetchone()
        conn.close()
        if row is not None:
            is_deprecated = bool(row[0])
            return tag_lookup, is_deprecated
        else:
            # Tag nie istnieje
            return tag_lookup, None
    except Exception as e:
        print(_("Błąd podczas odczytu tags:"), e)
        return tag_lookup, None
